@@ -271,8 +271,8 @@ class InterpretationCallback(Callback):
         targets = state.input['targets']
         overall_res = self.overall_loss(outputs, targets)
         start = self.loader_counts[state.loader_name]
-        bs = len(any_value(outputs))
 
+        n = 0
         for path, loss in overall_res.items():
             if path.startswith('indices'):
                 continue
@@ -280,19 +280,25 @@ class InterpretationCallback(Callback):
             ind_key = f'indices|{path}'
             indices = overall_res[ind_key] + start
             self.interpretations[state.loader_name][ind_key].append(indices.detach().cpu().numpy())
-        self.loader_counts[state.loader_name] += bs
+            n = len(indices)
+
+        self.loader_counts[state.loader_name] += n
 
     def on_loader_end(self, state: State):
         if should_skip_loader(state, self.loaders_to_skip):
             return
-        self.interpretations[state.loader_name] = {
-            key: np.concatenate(value, axis=0)
-            for key, value in self.interpretations[state.loader_name].items()
-        }
+        self.interpretations[state.loader_name] = self.prepare_interpretations(state)
 
         if self.tensorboard_converters is not None:
             self.tensorboard_converters.publish(state, self.interpretations[state.loader_name])
 
+    def prepare_interpretations(self, state):
+        res = {}
+        for key, value in self.interpretations[state.loader_name].items():
+            value = np.concatenate([arr for arr in value if len(arr) > 0])
+            res[key] = value
+        return res
+
     def on_stage_end(self, state: State):
         if should_skip_loader(state, self.loaders_to_skip):
             return
