@@ -1,5 +1,5 @@
 from dataclasses import dataclass, field
-from typing import Dict
+from typing import Dict, Optional
 
 import torch
 from torch import nn
@@ -68,22 +68,22 @@ class ModuleDecorator(nn.Module):
         self.module = task.torch()
         self.activation = task.get_activation()
         self.decoder = task.get_decoder()
+        self.dropout_mc = task.get_dropout_mc()
 
     def forward(self, *args, **kwargs):
-        if self.training:
-            args, kwargs, gt = find_gt_and_process_args_when_training(*args, **kwargs)
-            decoded_logits = gt.get(self.task_name, None)
-        else:
-            decoded_logits = None
-
-        logits = self.module(*args, **kwargs)
-        activated_logits = self.activation(logits) if self.activation is not None else logits
-
-        if decoded_logits is None:
-            decoded_logits = self.decoder(activated_logits) if self.decoder is not None else activated_logits
         key = self.prefix + self.task_name
+        logits = self.module(*args, **kwargs)
         condition = OnesCondition(key)
-        return LeafModuleOutput(key, logits, activated_logits, decoded_logits, condition)
+        if self.training:
+            return LeafModuleOutput(path=key, logits=logits, precondition=condition,
+                                    activated=None, decoded=None, dropout_samples=None)
+        activated_logits = self.activation(logits) if self.activation is not None else logits
+        decoded_logits = self.decoder(activated_logits) if self.decoder is not None else activated_logits
+        condition = OnesCondition(key)
+        dropout_samples = None if self.dropout_mc is None else self.dropout_mc.create_samples(self.module,
+                                                                                              *args, **kwargs)
+        return LeafModuleOutput(path=key, logits=logits, precondition=condition,
+                                activated=activated_logits, decoded=decoded_logits, dropout_samples=dropout_samples)
 
 
 class Condition(ICondition):
@@ -174,15 +174,17 @@ class AndCondition(Condition):
 class LeafModuleOutput(IFlowTaskResult):
     path: str
     logits: torch.Tensor
-    activated: torch.Tensor
-    decoded: torch.Tensor
     precondition: Condition
+    activated: Optional[torch.Tensor]
+    decoded: Optional[torch.Tensor]
+    dropout_samples: Optional[torch.Tensor]
 
     def add_to_composite(self, composite_module_output):
         composite_module_output.logits[self.path] = self.logits
         composite_module_output.activated[self.path] = self.activated
         composite_module_output.decoded[self.path] = self.decoded
         composite_module_output.preconditions[self.path] = self.precondition
+        composite_module_output.dropout_samples[self.path] = self.dropout_samples
 
     def __or__(self, precondition: Condition):
         """
@@ -209,6 +211,8 @@ class CompositeModuleOutput(IFlowTaskResult):
     activated: Dict[str, torch.Tensor] = field(default_factory=lambda: {})
     decoded: Dict[str, torch.Tensor] = field(default_factory=lambda: {})
     preconditions: Dict[str, Condition] = field(default_factory=lambda: {})
+    # A number of predictions for the same tasks, with different parts dropped out
+    dropout_samples: Dict[str, torch.Tensor] = field(default_factory=lambda: {})
 
     def add_to_composite(self, other):
         _copy_to_self(self.logits, other.logits)
@@ -242,23 +246,24 @@ class CompositeModuleOutput(IFlowTaskResult):
         return self
 
     def reduce(self):
-        if len(self.prefix) == 0:
-            inference_without_gt = self.gt is None
-            preconditions_source = self.decoded if inference_without_gt else self.gt
-            res = self.decoded if inference_without_gt else self.logits
+        if len(self.prefix) != 0:
+            # if this is not root, do not reduce yet.
+            return self
+        inference_without_gt = self.gt is None
+        preconditions_source = self.decoded if inference_without_gt else self.gt
+        res = self.decoded if inference_without_gt else self.logits
 
-            if inference_without_gt and not self.training:
-                for key, value in self.preconditions.items():
-                    if value is not None:
-                        self.preconditions[key] = value.to_mask(preconditions_source)
-                return self
+        if inference_without_gt and not self.training:
             for key, value in self.preconditions.items():
                 if value is not None:
-                    res[f'precondition|{key}'] = value.to_mask(preconditions_source)
-            if not inference_without_gt:
-                res['gt'] = self.gt
-            return res
-        return self
+                    self.preconditions[key] = value.to_mask(preconditions_source)
+            return self
+        for key, value in self.preconditions.items():
+            if value is not None:
+                res[f'precondition|{key}'] = value.to_mask(preconditions_source)
+        if not inference_without_gt:
+            res['gt'] = self.gt
+        return res
 
 
 @dataclass
